//INTERACTIVE VISUALIZATION OF PHOTONIC RUTHERFORD SCATTERING, Markus Selmke (selmke@rz.uni-leipzig.de, markus.selmke@gmx.de)

import controlP5.*;
ControlP5 controlP5;

/* @pjs preload="BG.jpg,Scan_RedGreen.png,Scan_Red.png,Scan_Red_PD.png,AuLens.png,AuLens_small.png,AuNP.png,AuNP_small.png,Zoom_Red.png,Zoom_Green.png,Zoom.png,PTSignalFormulaw.png,PTSignalFormulaPDw.png,PTSignalFormuladzfw.png,GreenLasers.png,RedLasers.png,Red_Beam_PT.png,Red_Beam_PTExpanded.png,Red_Beam_PTContracted.png,Signal0.png"; */

PImage imBG, imTr, imnr, imT0, imn0, imIdz, imIhz, imAuNP, imAuLens, imGreenLaser, imRedLaser;
PImage im_f, im_feff, im_dndT, im_DT, im_w0d, im_bbyR, im_R, im_Rnm, im_w0dnm, im_b, im_rmin, im_n2RF, im_n2;
PImage im_fb, im_phi0, im_pbyb, im_bxi, im_gamma, im_theta, im_rminb;
PImage destination, source;
public int Pred, Radius=40, NPAlpha, Show_NPPlot=1, x, y, i=1, strX=10, n2alpha;
String strParam0 = "", strParam1 = "", strParam2 = "", strParam3 = "", strParam4 = "", strParam5 = "", strParam6 = ""; 
public float T0=25.0, DT=200.0, n0=1.46, dndT=-3.6, b=2.0, DTMax=200.0, nScale=1000, TScale=0.6, n0P=1.3, NRays=70, db=20, wd=200;
public float ecc, ecc1, ecc2, xi=0.0, rmin=0.0, p=0.0, gam=0.0, phi=0.0, phi0=0.0, bxi=0.0, r=0.0, thetaScat=0.0, epsPhi=PI/200, Exaggeration=20.0, f=0.0, bi, AmpG, feff=0.0;
public float zR=PI*wd*wd/632.0, zp=0.0;
int bh=20, bw=200, bx=40, Tol=50, TrPlotExtend=300, ArrL=12, ArrW=7;
public PFont font, CP5font;
public boolean ManyTrajToggle, GenTrajToggle, SingleTrajToggle, DrawShadowToggle, DrawRFShadowToggle, RFScattToggle;
public boolean CheckerboardToggle, NetToggle, QuadsToggle, DoVideo=true, VideoToggle, CameraExists=false, SquaresToggle;
public float[] b_vec = new float[70];

int BoxW=15, dx, dy;
public float phiXY, z=-2.2*Radius, X1, Y1, X2, Y2, X3, Y3, X4, Y4, y1, y2, y3, y4, b1, b2, b3, b4, yn;
float cr, cg, cb;
int xv, yv;

void setup() { 
  zR=1000000000; //parallel gaussian beam
  size(1000, 800);
  smooth();
  frameRate(27);
  

  CameraExists = false;
  
  // The background image must be the same size as the parameters into the size() method.
  // is 800 x 800 pixels. Load all used images.
  imBG = loadImage("BG.jpg");
  imAuLens = loadImage("AuLens.png");
  imAuNP = loadImage("AuNP.png");
  imTr = loadImage("Tr.png");
  imnr = loadImage("nr.png");
  imn0 = loadImage("n0.png");
  imT0 = loadImage("T0.png");
  imIhz = loadImage("Ihz.png");
  imIdz = loadImage("Idz.png");
  
  //Grid = loadImage("Grid.png");
  destination = createImage(imBG.width, imBG.height, ARGB);
  
  im_f = loadImage("formula_f.png");
  im_feff = loadImage("formula_feff.png");
  im_dndT = loadImage("formula_dndT.png");
  im_DT = loadImage("formula_DT.png");
  im_w0d = loadImage("formula_w0d.png");
  im_bbyR = loadImage("formula_bbyR.png");
  im_R = loadImage("formula_R.png");
  im_Rnm = loadImage("formula_Rnm.png");
  im_w0dnm = loadImage("formula_w0dnm.png");
  im_rmin = loadImage("formula_rmin.png");
  im_b = loadImage("formula_b.png");
  im_n2RF = loadImage("formula_n2RF.png");
  im_n2 = loadImage("formula_n2.png");
  im_fb = loadImage("formula_fb.png");
  im_phi0 = loadImage("formula_phi0.png");
  im_pbyb = loadImage("formula_pbyb.png");
  im_bxi = loadImage("formula_bxi.png");
  im_gamma = loadImage("formula_gamma.png");
  im_theta = loadImage("formula_theta.png");
  im_rminb = loadImage("formula_rminb.png");
  
  imGreenLaser = loadImage("GreenLasers.png");
  imRedLaser = loadImage("RedLasers.png");
  
  font = createFont("Arial", 12); 
  textFont(font);
  
  //initialize control sliders and checkboxes
  controlP5 = new ControlP5(this);
  
    ControlFont CP5font = new ControlFont(font); 
    controlP5.setControlFont(CP5font);
    
    Slider DTslider = controlP5.addSlider("DT", 0, DTMax, 0, 20, 720, bw, bh);
      DTslider.setValue(DT);
    Slider Exaggerationslider = controlP5.addSlider("exaggeration", 1.0, 40.0, 0, 820, 720, bw/2, bh);
      Exaggerationslider.setValue(Exaggeration);
    Slider dndTslider = controlP5.addSlider("dndT", -5.1, 5.1, 0, 20, 690, bw, bh);
    Slider bslider = controlP5.addSlider("impactparameter", -9.0, 9.0, 0, 20, 660, bw, bh);
    Slider wdslider = controlP5.addSlider("beamwaist", 50.0, 800.0, 0, 20, 630, bw, bh);
      wdslider.setValue(wd);
    Toggle ManyTrajectories = controlP5.addToggle("ManyTraj", ManyTrajToggle, 20, 580, 20, 20);
      ManyTrajectories.setValue(false);
    Toggle SingleTrajectory = controlP5.addToggle("SingleTraj", SingleTrajToggle, 80, 580, 20, 20);
      SingleTrajectory.setValue(true);
    Toggle DrawShadow = controlP5.addToggle("Shadow", DrawShadowToggle, 880, 550, 20, 20);
      DrawShadow.setValue(false);
    Toggle DrawRFShadow = controlP5.addToggle("RFShadow", DrawRFShadowToggle, 930, 550, 20, 20);
      DrawRFShadow.setValue(false);
    Toggle RFScatt = controlP5.addToggle("RFScatt", RFScattToggle, 820, 550, 20, 20);
      RFScatt.setValue(false);
    
    Toggle Net = controlP5.addToggle("Net", NetToggle, 970, 610, 20, 20);
      Net.setValue(false);
    Toggle Checkerboard = controlP5.addToggle("Checkerboard", CheckerboardToggle, 920, 610, 20, 20);
      Checkerboard.setValue(false);
    Toggle Quads = controlP5.addToggle("Quads", QuadsToggle, 870, 610, 20, 20);
      Quads.setValue(false);
    Toggle Squares = controlP5.addToggle("Squares", SquaresToggle, 820, 610, 20, 20);
      Squares.setValue(false);
    Slider BWidthSlider = controlP5.addSlider("boxwidth", 3, 35, 0, 820, 690, bw/2, bh);
      BWidthSlider.setValue(BoxW);
    Slider zImagePosSlider = controlP5.addSlider("zImagePos", -10.0, 0, 0, 820, 660, bw/2, bh);
      zImagePosSlider.setValue(z/Radius);
    
    dndTslider.setNumberOfTickMarks(3);
    dndTslider.showTickMarks(true);
    dndTslider.snapToTickMarks(false);
    //dndTslider.setLabelVisible(false);
    dndTslider.setValue(-3.6);
    
    bslider.setNumberOfTickMarks(3);
    bslider.showTickMarks(true);
    bslider.snapToTickMarks(false);
    bslider.setValue(2.0);
    
    //change their apperance!
    dndTslider.captionLabel().toUpperCase(false);
    Exaggerationslider.captionLabel().toUpperCase(false);
    bslider.captionLabel().toUpperCase(false);
    DTslider.captionLabel().toUpperCase(false);
    wdslider.captionLabel().toUpperCase(false);
    ManyTrajectories.captionLabel().toUpperCase(false);
    SingleTrajectory.captionLabel().toUpperCase(false);
    Checkerboard.captionLabel().toUpperCase(false);
    Net.captionLabel().toUpperCase(false);
    BWidthSlider.captionLabel().toUpperCase(false);
    Quads.captionLabel().toUpperCase(false);
    Squares.captionLabel().toUpperCase(false);
    zImagePosSlider.captionLabel().toUpperCase(false);
    DrawShadow.captionLabel().toUpperCase(false);
    DrawRFShadow.captionLabel().toUpperCase(false);
    RFScatt.captionLabel().toUpperCase(false);
    
    RFScatt.setCaptionLabel("phot. RF");
    DrawShadow.setCaptionLabel("shadow");
    DrawRFShadow.setCaptionLabel("RF shadow");
    Squares.setCaptionLabel("squares");
    zImagePosSlider.setCaptionLabel("z(Image)/R");
    Quads.setCaptionLabel("quads");
    Net.setCaptionLabel("net");
    BWidthSlider.setCaptionLabel("pattern size");
    Checkerboard.setCaptionLabel("checker");
    DTslider.setCaptionLabel("temperature elev.:");
    DTslider.setColorBackground(color(255, 0, 0));
    bslider.setCaptionLabel("impact parameter:");
    wdslider.setCaptionLabel("det. beam waist:");
    dndTslider.setCaptionLabel("thermorefr. coef:");
    ManyTrajectories.setCaptionLabel("many traj.");
    SingleTrajectory.setCaptionLabel("annot. traj.");
    Exaggerationslider.setCaptionLabel("exaggeration");
}

void boxwidth(int theValue) {
  BoxW=theValue;
}

void impactparameter(float theValue) {
  //if (abs(theValue) > 1) {
    b=theValue*Radius;
  //} else {
    //b=1.02*Radius;
  //}
}

void red_laser_power(int theValue) {
  Pred=theValue;
}

void beamwaist(int theValue) {
  wd=theValue;
}

void DT(float theValue) {
  DT=theValue;
}

void exaggeration(float theValue) {
  Exaggeration=theValue;
}

void dndT(float theValue) {
  dndT=theValue*0.0001;
}

void zImagePos(float theValue) {
  z=theValue*Radius;
}

void ManyTraj(boolean theFlag) {
  ManyTrajToggle=theFlag;
  GenTrajToggle=theFlag;
}

void SingleTraj(boolean theFlag) {
  SingleTrajToggle=theFlag;
}

void Shadow(boolean theFlag) {
  DrawShadowToggle=theFlag;
}

void RFShadow(boolean theFlag) {
  DrawRFShadowToggle=theFlag;
}

void RFScatt(boolean theFlag) {
  RFScattToggle=theFlag;
  n2alpha = 255;
}

void Checkerboard(boolean theFlag) {
  CheckerboardToggle=theFlag;
  
  if (CheckerboardToggle==true) {
    controlP5.controller("Quads").setValue(0);
    controlP5.controller("Squares").setValue(0);
  }
}

void Net(boolean theFlag) {
  NetToggle=theFlag;
  
  if (NetToggle==true) {
    controlP5.controller("Quads").setValue(0);
  }
}

void Squares(boolean theFlag) {
  SquaresToggle=theFlag;
  
  if (SquaresToggle==true) {
    controlP5.controller("Quads").setValue(0);
    controlP5.controller("Checkerboard").setValue(0);
  }
}

void Quads(boolean theFlag) {
  QuadsToggle=theFlag;
  
  if (QuadsToggle==true) {
    controlP5.controller("Net").setValue(0);
    controlP5.controller("Checkerboard").setValue(0);
    controlP5.controller("Squares").setValue(0);
  }
}

void draw() 
{
  //set background image, load font, write title
    frame.setTitle("PHOTONIC RUTHERFORD SCATTERING");
    background(imBG);

  //show formula of actual n(r):
    n2alpha = round(n2alpha - 3);
    tint(255,n2alpha);
    if (RFScattToggle == true) {  
      image(im_n2RF, 650, 20);
    } else {
      image(im_n2, 650, 20);
    }

    //DRAW DISTORTION FIELD
    if (CheckerboardToggle==true || NetToggle==true || QuadsToggle==true || SquaresToggle==true) {
      DoVideo=false;
      //noTint();
      //image(destination,0,0);
      strokeWeight(1);
      if ((NetToggle==true || CheckerboardToggle==true) && SquaresToggle==false) {
        dx = BoxW; dy=2*BoxW; //displays a distorted net
      } else if (QuadsToggle==true) {
        dx = 2*BoxW; dy=2*BoxW; //quads ordered uniformly spaced by another quad
      } else if (SquaresToggle==true) {
        dx=BoxW; dy=BoxW;
      }
      for (int x = -2*dx; x < 1000+2*dx; x=x+dx) {
       for (int y = 0-2*dy; y < 800+2*dy; y=y+dy) {
         //starting from top left corner (x,y) or (x,yn) do quad
         if (CheckerboardToggle==true || NetToggle==true) {
           yn=((x/dx)%2)*BoxW+y; //vertically shift every second column by a box
         } else {
           yn = y;
         }
         //get corner coordinates and warp them
           phiXY = atan2(yn-400,x-500); b1 = sqrt((x-500)*(x-500)+(yn-400)*(yn-400)); getParms(b1);
           y1 = b1 + tan(thetaScat)*(z + b1*tan(phi0-PI/2));
           X1 = 500 + cos(phiXY)*y1; Y1 = 400 + sin(phiXY)*y1;
           phiXY = atan2(yn-400,x+BoxW-500); b2 = sqrt((x+BoxW-500)*(x+BoxW-500)+(yn-400)*(yn-400)); getParms(b2);
           y2 = b2 + tan(thetaScat)*(z + b2*tan(phi0-PI/2));
           X2 = 500 + cos(phiXY)*y2; Y2 = 400 + sin(phiXY)*y2;
           phiXY = atan2(yn+BoxW-400,x-500); b3 = sqrt((x-500)*(x-500)+(yn+BoxW-400)*(yn+BoxW-400)); getParms(b3);
           y3 = b3 + tan(thetaScat)*(z + b3*tan(phi0-PI/2));
           X3 = 500 + cos(phiXY)*y3; Y3 = 400 + sin(phiXY)*y3;
           phiXY = atan2(yn+BoxW-400,x+BoxW-500); b4 = sqrt((x+BoxW-500)*(x+BoxW-500)+(yn+BoxW-400)*(yn+BoxW-400)); getParms(b4);
           y4 = b4 + tan(thetaScat)*(z + b4*tan(phi0-PI/2));
           X4 = 500 + cos(phiXY)*y4; Y4 = 400 + sin(phiXY)*y4;
         //original quad, only in quad mode
           if (QuadsToggle==true) {
             stroke(255,50+((b1-y1)*(b1-y1)));
             noFill();
             quad(x,yn,x+BoxW,yn,x+BoxW,yn+BoxW,x,yn+BoxW);
           }
         //warped quad
           if (NetToggle==true) {
             if (CheckerboardToggle==true || SquaresToggle==true) {
               stroke(0,50+((b1-y1)*(b1-y1))); //black
             } else {
               stroke(255,50+((b1-y1)*(b1-y1))); //white  
             }
           } else {
             noStroke();
           }
           //fill(255,255,0,50+((b1-y1)*(b1-y1)));
           if (CheckerboardToggle==true || QuadsToggle==true || SquaresToggle==true) {
             fill(255,255,0,50+((b1-y1)*(b1-y1))); //standard yellow semi-transparent filling
           } else {
             noFill();
           }
           if (y1>0 && y2>0 && y3>0 && y4>0) {
             quad(X1,Y1,X2,Y2,X4,Y4,X3,Y3);
           }
       }
     }
     //Draw line at image plane position
     if (ManyTrajToggle==true) {
        strokeWeight(6); stroke(20,255); fill(20,200);
        line(500+z,0,500+z,800);
        rotate(PI/2);
          text("image position", 25, -508-z);
        rotate(-PI/2); 
      }
    } //END DISTORTION FIELD
    
  //horizontal axis
  if (SingleTrajToggle==true || ManyTrajToggle==true) {
      strokeWeight(2); stroke(255,100); 
      line(0,400,1000,400); 
      fill(255); 
      text("optical axis", 500+TrPlotExtend*0.75, 400+15);
  }
      text("click on NP to show / hide Profiles!", 10, 12);
    
  //Draw cold AuNP  
      tint(255,255);
      image(imAuNP, -20+100, 2);
    
  //baseline for T(r) and n(r)  
    if (Show_NPPlot==1) {
      strokeWeight(2); stroke(255); 
      line(500-TrPlotExtend,400,500+TrPlotExtend,400); //horizontal axis
      line(500,400,500,400-TrPlotExtend); //vertical axis
      triangle(500,400-TrPlotExtend-ArrL,500+ArrW/2,400-TrPlotExtend,500-ArrW/2,400-TrPlotExtend);
      triangle(500+TrPlotExtend+ArrL,400,500+TrPlotExtend,400+ArrW/2,500+TrPlotExtend,400-ArrW/2);
    }
    
  //actual big Particle images, i.e. AuNP and Hot AuNP with Lens
    tint(255, 255.0*DT/DTMax);
    image(imAuLens, -20+100, 2); 
    //annotate AuNP
      NPAlpha = round(255.0*exp(-((mouseX-400)*(mouseX-400) + (mouseY-400)*(mouseY-400))/1000.0));
      fill(0, NPAlpha);
      text("gold (Au)", 375, 400);
      text("nano-particle", 367, 410);
    
  //PLOT T(r) & n(r) Profile
    noFill();
    if (Show_NPPlot==1) {
      tint(255, 255);
      noFill(); strokeWeight(2); strokeJoin(ROUND); 
      stroke(200,0,0); 
      beginShape();
      for (x=-TrPlotExtend; x<=TrPlotExtend; x=x+2) {
        if (abs(x) > Radius) {
          vertex(500+x,400 - TScale*(T0 + DT*Radius/abs(x)));
        } else {
          vertex(500+x,400 - TScale*(T0 + DT));
        }
      }  
      endShape();
      image(imTr, 600, 400 - 20 - TScale*(T0 + DT*Radius/100)); //png: T(r)
      //Refractive Index Profile
      stroke(0,200,200);
      beginShape();
      for (x=-TrPlotExtend; x<=TrPlotExtend; x=x+2) {
        if (abs(x) > Radius) {
          vertex(500+x,400 + nScale*dndT*DT - nScale*(n0 - n0P + dndT*DT*Radius/abs(x)));
        } else {
          vertex(500+x,400 + nScale*dndT*DT - nScale*(n0 - n0P + DT*dndT));
        }
      }  
      endShape();
      //display formulas: T_0, n_0, n(r)
      image(imnr, 600, 400 - 30 + nScale*dndT*DT - nScale*(n0 - n0P + dndT*DT*Radius/100)); //png: T(r)
      image(imn0, 520, 400 - 20 - nScale*(n0 - n0P));
      image(imT0, 480, 400 - 20 - TScale*(T0+DT));
    }
    stroke(255,255);
    line(500-Radius,400-5,500-Radius,400+5);
    line(500+Radius,400-5,500+Radius,400+5);
    image(im_R,500+Radius+2,400+10);
    
    
    //=============================================================
    //DRAW SHADOW REGION
      if (DrawShadowToggle==true) {
        strokeWeight(2); strokeJoin(ROUND); stroke(0,255,0,100);
        beginShape();
        for (X1=0; X1<=500; X1=X1+1) {
           Y1 = pow(2,1.5)*sqrt(X1+1/xi)/sqrt(xi) - PI*(2+X1*xi)/(4*(xi+X1*xi*xi)); //more exact
           Y1 = pow(2,1.5)*sqrt(X1/xi) - PI/(4*xi); //less accurate
           Y1 = pow(2,1.5)*sqrt(X1+1/xi)/sqrt(xi); //RF
           vertex(500 + X1,400 - Y1);
        }
        for (X1=500; X1>=0; X1=X1-1) {
           Y1 = pow(2,1.5)*sqrt(X1+1/xi)/sqrt(xi) - PI*(2+X1*xi)/(4*(xi+X1*xi*xi)); //more exact
           Y1 = pow(2,1.5)*sqrt(X1/xi) - PI/(4*xi); //less accurate
           Y1 = pow(2,1.5)*sqrt(X1+1/xi)/sqrt(xi); //RF
           vertex(500 + X1,400 + Y1);
        }  
        endShape(); 
      }
      if (DrawRFShadowToggle == true) {
        strokeWeight(2); strokeJoin(ROUND); stroke(20,255,100,100); fill(20,255,100,100);
        beginShape();
        for (phi=2*PI-epsPhi; phi>=0+10*epsPhi; phi=phi-PI/200) {
           float rShadow = (4/xi)/(1 - cos(phi));
           vertex(500 + rShadow*cos(phi),400 + rShadow*sin(phi));
        }  
        endShape(); 
      }
    
    //PLOT TRAJECTORY & get Trajectory parameters
      getParms(abs(b));
      
    if (SingleTrajToggle==true) {
      //plot parameters
      strParam0 = "= " + nf(bxi,1,3);
      strParam1 = "= " + nf(thetaScat*180/PI,1,3) + "°";
      strParam2 = "= " + nf(phi0*180/PI,1,3) + "°";
      strParam3 = "= " + nf(p/b,1,3);
      //semi latus rectum is p
      strParam4 = "= " + nf(gam,1,3);
      strParam5 = "= " + nf(f,1,3);
      strParam6 = "     = " + nf(rmin,1,3);
      fill(255);
      text(strParam0,45,30+strX); text(strParam1,45,45+strX); text(strParam2,45,60+strX);
      text(strParam3,45,75+strX); text(strParam4,45,90+strX); text(strParam5,45,105+strX);
      text(strParam6,45,120+strX);
      noFill();
      
      tint(255, 255);
      image(im_bxi,5,30-2);
      image(im_theta,5,45-2);
      image(im_phi0,5,60-2);
      image(im_pbyb,5,75-2);
      image(im_gamma,5,90-2);
      image(im_fb,5,105-2);
      image(im_rminb,5,120-2);
    
    strokeWeight(2); stroke(255,165,0,255); 
      line(500+f,400+15,500+f,400-15); //mark focal length
      image(im_f,500+f,400+20);
    strokeWeight(2); stroke(255,165,0,100); 
      line(0,400+b,1000,400+b); //asymptote 1
    //asymptote 2:
    if (bxi>0) {
      if (b > 0) {
        line(500+f*1.1,400+b+tan(thetaScat)*(f*1.1+b*tan(phi0-PI/2)),1000,400+b+tan(thetaScat)*(500+b*tan(phi0-PI/2))); 
      } else {
        line(500+f*1.1,400+b-tan(thetaScat)*(f*1.1-b*tan(phi0-PI/2)),1000,400+b-tan(thetaScat)*(500-b*tan(phi0-PI/2)));
      }  
    } else {
      if (b > 0) {
        line(500,400+b+tan(thetaScat)*(b*tan(phi0-PI/2)),1000,400+b+tan(thetaScat)*(500+b*tan(phi0-PI/2))); 
      } else {
        line(500,400+b-tan(thetaScat)*(-b*tan(phi0-PI/2)),1000,400+b-tan(thetaScat)*(500-b*tan(phi0-PI/2)));
      }
    }
    //plot angle of closest approach
    strokeWeight(2); stroke(255,0,0,100);
      if (RFScattToggle==true) {
        r = 1/xi + sqrt(b*xi*b*xi+1)/xi; //=r(phi0)=p/(bxi-1)
      } else {
        r = abs(b) + 1/xi; //=r(phi0)=p/(bxi-1)
      }
      rmin = abs(r/b);
      line(500,400,500+r*cos(phi0),400+signum(b)*r*sin(phi0));
      ellipseMode(CENTER); fill(255,0,0,200);
      ellipse(500+r*cos(phi0),400+signum(b)*r*sin(phi0),7,7);
      ellipse(500,400,7,7);
      image(im_rmin,500+r*cos(phi0)/2.0,400+signum(b)*r*sin(phi0)/2.0+signum(b)*20-5);
    //point P for asymptote  
      ellipse(500-abs(b)*tan(phi0-PI/2),400+b,5,5);
    //plot b
      stroke(255,255); strokeWeight(1); fill(255);
      line(20,400,20,400+b); 
      triangle(20-ArrW/2,400+b-signum(b)*ArrL,20+ArrW/2,400+b-signum(b)*ArrL,20,400+b);
      image(im_b,27,400+b/2.0);
    
    //Plot Single Ray, <<<RED CONTROLABLE RAY>>>
      stroke(255,165,0); strokeWeight(3); strokeJoin(ROUND); noFill();
      beginShape();
      for (phi=PI-epsPhi; phi>=thetaScat+epsPhi; phi=phi-PI/200) {
         r = getRadius(phi);
         vertex(500 + r*cos(phi),400 + signum(b)*r*sin(phi));
      }  
      endShape();
    }
    
    if (ManyTrajToggle==true) {
      //get effective focal length for offset gaussian beam
        getParms(wd*sqrt((zp*zp/(zR*zR) + 1))/sqrt(2)); //i.e. feff=f(wd(zp))
        feff = f;
      
      //PLOT BEAM ON THE SIDE OF THE SCREEN
        noFill();
        strokeWeight(4);
        for (x=-400; x<=400; x=x+2) {
          stroke(255,0,0,255.0*exp(-2*x*x/(wd*wd)));
          line(0,400+x,10,400+x);  
        }
        stroke(255,0,0,255);
        line(0,400+wd,20,400+wd);
        line(0,400-wd,20,400-wd);
        image(im_w0d,30,400-wd);
      
      //mark effective focal length
        stroke(255,165,0,255); strokeWeight(1);
        line(500+feff,400+30,500+feff,400-30);
        image(im_feff,500+feff,400+20);
      
      //generate random impact parameters
      if (GenTrajToggle==true) {
        for (i=0; i<NRays; i=i+1) {
          b_vec[i] = 400*randomNormal();
        }
        GenTrajToggle=false;
      }
      
      //plot corresponding trajectories, i.e. go through b_vec[] vector of impact parameters
      for (i=0; i<NRays; i=i+1) {
        bi = b_vec[i];
        getParms(abs(bi)); //get trajectory parameters
        //transparency of the trajectories according to gaussian beam
          AmpG = 255*exp(-2*bi*bi/(wd*wd));
          stroke(250,128,114,AmpG); strokeWeight(2); 
          line(500+f,400+5,500+f,400-5); //mark focal length
        //COLORIZE THE INDIVIDUAL RAYS
          stroke(220,0,0,AmpG);
        strokeWeight(1); strokeJoin(ROUND); 
        beginShape();
        for (phi=PI-epsPhi; phi>=thetaScat+epsPhi; phi=phi-PI/200) {
          r = getRadius(phi);
          vertex(500 + r*cos(phi),400 + signum(bi)*r*sin(phi));
        }  
        endShape(); 
      }
    }
    
    tint(255, 255);
    image(im_w0dnm,317,630);
    image(im_bbyR,330,660);
    image(im_dndT,320,690);
    image(im_DT,331,720);
}

float getRadius(float phi) {
  float radius;
  if (RFScattToggle==true) {
      radius = p/(signum(xi)*ecc*cos(phi-phi0) - 1);
  } else {
    if (abs(bxi) > 1) {
      radius = p/(ecc*cos(gam*(phi-phi0)) - 1);
    } else {
      radius = p/(ecc1*exp(gam*phi)+ecc2*exp(-gam*phi) + 1);
    //if (xi > 0) {
      //float coshargument = gam*(phi-phi0);
      //float cosh = (exp(coshargument) + exp(-coshargument))/2;
      //radius = p/(ecc*cosh + 1);
      //radius = p/(ecc1*exp(gam*phi)+ecc2*exp(-gam*phi) + 1);
    //} else {
      //float sinhargument = gam*(phi-phi0);
      //float sinh = (exp(sinhargument) - exp(-sinhargument))/2;
      //radius = p/(ecc*sinh + 1);
      //radius = p/(ecc1*exp(gam*phi)+ecc2*exp(-gam*phi) + 1);
    //}    
    }
  }
  return radius;
}

float getParms(float b) {
      xi = -n0/(Radius*dndT*DT*Exaggeration); //inverse strength
      bxi = b*xi; //b*xi=epsilon
      
      if (abs(bxi) > 1) {
        ecc = bxi; //eccentricity
        p = (bxi*bxi - 1)/xi; //parameter p
        gam = sqrt(1 - 1/(bxi*bxi)); //perturbation parameter gam
        
        //phi0 = PI - bxi*atan(sqrt(bxi*bxi-1))/sqrt(bxi*bxi-1);
        phi0 = PI - acos(1/bxi)/gam;
        f = -b/sin(2*acos(1/bxi)/gam);
      } else {
        p = (1 - bxi*bxi)/xi; //parameter, p_r and p_a
        gam = sqrt(1/(bxi*bxi) - 1); //perturbation parameter, gam_r and gam_s
        
        if (xi > 0) { //EXREME REPULSIVE
          ecc = -bxi;
          float acosh = log(1/bxi + sqrt(1/bxi + 1)*sqrt(1/bxi - 1)); //arccosh(z) = ln(z+sqrt(1+z)*sqrt(1-z)), http://mathworld.wolfram.com/InverseHyperbolicCosine.html
          phi0 = PI - acosh/gam;
          f = -b/sin(2*acos(1/bxi)/gam);
          
          float eccParm = (1 + sqrt(1-bxi*bxi))/2;
          ecc1 = -exp(-PI*gam)*eccParm;
          ecc2 = -exp(PI*gam)*(1 - eccParm);
        } else { //EXREME ATTRACTIVE
          //ecc = -2.0/(exp(PI*gam) - exp(-PI*gam));
          //ecc = bxi;
          //float asinh = log(1/ecc + sqrt(1 + 1/(ecc*ecc))); //arcsinh(z) = ln(z+sqrt(1+z^2)), http://mathworld.wolfram.com/InverseHyperbolicSine.html
          //phi0 = PI + asinh/gam;
          float eccParm = (1 - sqrt(1-bxi*bxi))/2;
          ecc1 = -exp(-PI*gam)*eccParm;
          ecc2 = -exp(PI*gam)*(1 - eccParm);
          
          f = 0;
        }
      }
      
      //scattering angle
      thetaScat = 2*phi0 - PI;
      
      if (RFScattToggle==true) {
        ecc = sqrt(bxi*bxi+1); 
        gam = 1; 
        p = xi*b*b;
        thetaScat=2*atan(1/(b*xi)); 
        phi0=(thetaScat+PI)/2; 
        f = -b*(1/tan(thetaScat)+tan(phi0-PI/2));
      }
      
      return 2;
}

//Toggle Zoom-Image
void mousePressed() {
  
  //plot bigger Graph original small plot at x=602, y=145-round(2*zp*30)
  if (Show_NPPlot == 1) {
    if (mouseX > 500-Radius && mouseX < 500+Radius && mouseY > 400-Radius && mouseY < 400+Radius) {
      Show_NPPlot = 0;
    }
  } else {
    if (mouseX > 500-Radius && mouseX < 500+Radius && mouseY > 400-Radius && mouseY < 400+Radius) {
      Show_NPPlot = 1;
    }
  }

}

// return a random value in the range [-1..1] with normal distribution around 0.
// Implements the Marsaglia Polar Method, as described in wikipedia, but only returns one of the values.
float randomNormal()
{
  float x = 1.0, y = 1.0, 
        s = 2.0; // s = x^2 + y^2
  while(s >= 1.0)
  {
    x = random(-1.0f, 1.0f);
    y = random(-1.0f, 1.0f);
    s = x*x + y*y;
  }
  return x * sqrt(-2.0f * log(s)/s);
}

float signum(float f) {
  if (f > 0) return 1.0;
  if (f < 0) return -1.0;
  return 0;
} 

//warp arbitrary source image, put into destination image
PImage warp2(PImage source) {
   float bb = 20.0, z = 20.0, phiXY=0.0, yy=0.0;
   int w = source.width, h = source.height;
   int newX, newY;
   PImage destination = new PImage(w,h);
   source.loadPixels();
   destination.loadPixels();

   for (int x = 0; x < w; x++) {
     for (int y = 0; y < h; y++) {
       phiXY = atan2(1.0*y-h/2,1.0*x-w/2);
       bb = sqrt((x-w/2)*(x-w/2)+(y-h/2)*(y-h/2));
       getParms(bb);
       yy = bb + tan(thetaScat)*(z + bb*tan(phi0-PI/2));
       
       newX = w/2 + round(cos(phiXY)*yy);
       newY = h/2 + round(sin(phiXY)*yy);
       color c;
       if(newX >= w || newX < 0 || newY >= h || newY < 0) {
         c = color(255,255,255,0);
       } else {
         c = source.pixels[newY*w + newX];
         destination.pixels[y*w+x] = c;
       }
     }
   }
   return destination;
}
